package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kstarbound.defs.tile.TileDefinition

/**
 * Предоставляет доступ к чанку и его соседям
 *
 * В основном для использования в местах, где нужен не мир, а определённый чанк мира,
 * и при этом координаты проверяются относительно чанка и могут спокойно выйти за его пределы,
 * с желанием получить тайл из соседнего чанка
 */
open class TileView(
	override val pos: ChunkPos,
	open val center: ITileChunk?,

	open val right: ITileChunk?,
	open val top: ITileChunk?,
	open val topRight: ITileChunk?,
	open val topLeft: ITileChunk?,

	open val left: ITileChunk?,
	open val bottom: ITileChunk?,
	open val bottomLeft: ITileChunk?,
	open val bottomRight: ITileChunk?,
) : ITileChunk {
	override fun get(x: Int, y: Int): ITileState {
		if (x in 0 ..CHUNK_SIZE_FF) {
			if (y in 0 ..CHUNK_SIZE_FF) {
				return center?.get(x, y) ?: EmptyTileState
			}

			if (y < 0) {
				return bottom?.get(x, y + CHUNK_SIZE) ?: EmptyTileState
			} else {
				return top?.get(x, y - CHUNK_SIZE) ?: EmptyTileState
			}
		}

		if (x < 0) {
			if (y in 0 ..CHUNK_SIZE_FF) {
				return left?.get(x + CHUNK_SIZE, y) ?: EmptyTileState
			}

			if (y < 0) {
				return bottomLeft?.get(x + CHUNK_SIZE, y + CHUNK_SIZE) ?: EmptyTileState
			} else {
				return topLeft?.get(x + CHUNK_SIZE, y - CHUNK_SIZE) ?: EmptyTileState
			}
		} else {
			if (y in 0 ..CHUNK_SIZE_FF) {
				return right?.get(x - CHUNK_SIZE, y) ?: EmptyTileState
			}

			if (y < 0) {
				return bottomRight?.get(x - CHUNK_SIZE, y + CHUNK_SIZE) ?: EmptyTileState
			} else {
				return topRight?.get(x - CHUNK_SIZE, y - CHUNK_SIZE) ?: EmptyTileState
			}
		}
	}
}

class MutableTileView(
	pos: ChunkPos,
	override val center: IMutableTileChunk?,

	override val right: IMutableTileChunk?,
	override val top: IMutableTileChunk?,
	override val topRight: IMutableTileChunk?,
	override val topLeft: IMutableTileChunk?,

	override val left: IMutableTileChunk?,
	override val bottom: IMutableTileChunk?,
	override val bottomLeft: IMutableTileChunk?,
	override val bottomRight: IMutableTileChunk?,
) : TileView(pos, center, right, top, topRight, topLeft, left, bottom, bottomLeft, bottomRight), IMutableTileChunk {
	override fun get(x: Int, y: Int): IMutableTileState {
		return super<TileView>.get(x, y) as IMutableTileState
	}
}
